
use adventureworks
go

select * from HumanResources.Employee

create table tree (
 node int,  --char(10) not null,
 parent int --char(10)
);
 
insert into tree 
select EmployeeID, ManagerID 
from AdventureWorks.HumanResources.Employee;
 
create table stack (
 stack_top integer not null,
 node int not null, --char(10) not null,
 lft int,
 rgt int
);
go

create procedure AdjToNested
as
declare @lft_rgt int;
declare @max_lft_rgt int;
declare @current_top int;

set @lft_rgt = 2;
set @max_lft_rgt = 2 * (select count(*) from tree);
set @current_top = 1;

-- clear the stack
delete from stack;
--push the root
insert into stack 
select 1, node, 1, @max_lft_rgt
 from tree
 where parent is null;

-- delete rows from tree as they are used 
delete from tree where parent is null;

while @lft_rgt <= @max_lft_rgt - 1
 begin 
 if exists (select * from stack as s1, tree as t1
			where s1.node = t1.parent
			and s1.stack_top = @current_top)
   begin 
   insert into stack 
   select (@current_top + 1), min(t1.node), @lft_rgt, NULL
   from stack as s1, tree as t1
   where s1.node = t1.parent
   and s1.stack_top = @current_top
   
   delete from tree
   where node = (select node from stack
				 where stack_top = @current_top + 1)
   set @lft_rgt += 1;
   set @current_top += 1;
   end
 else
   begin 
     update stack set rgt = @lft_rgt,
                      stack_top = (stack_top * -1)
     where stack_top = @current_top
   set @lft_rgt += 1;
   set @current_top -= 1;  
   end
end

exec AdjToNested

-- check 
if exists (select * from tree)
  print ('nodes still left in tree');
go

select * into AdventureWorks..EmployeeNSet
from AdventureWorks.HumanResources.Employee  

select * from AdventureWorks..EmployeeNSet

alter table AdventureWorks..EmployeeNSet DROP COLUMN ManagerID
alter table AdventureWorks..EmployeeNSet ADD lft int
alter table AdventureWorks..EmployeeNSet ADD rgt int

UPDATE AdventureWorks..EmployeeNSet
SET lft = s.lft, rgt = s.rgt
FROM stack s
JOIN AdventureWorks..EmployeeNSet e
ON s.node = e.EmployeeID

-- these match by NationalIDNumber and LoginID and Title, but not by EmployeeID
select top (5) * from AdventureWorks..EmployeeNSet
order by LoginID

select top (5) * from AdventureWorks2008.HumanResources.Employee
order by LoginID

alter table AdventureWorks..EmployeeNSet ADD CONSTRAINT [PK_EmployeeNSet_EmployeeID] PRIMARY KEY CLUSTERED ([EmployeeID] ASC)

use AdventureWorks
go

create index EmployeeNSet_IdxLft on AdventureWorks..EmployeeNSet(lft)

create index EmployeeNSet_IdxRgt on AdventureWorks..EmployeeNSet(rgt)

create index EmployeeNSet_IdxLftRgt on AdventureWorks..EmployeeNSet(lft, rgt)


/* Add other constraints, if needed

ALTER TABLE AdventureWorks..EmployeeNSet  WITH CHECK ADD  CONSTRAINT [CK_Employee_BirthDate] CHECK  (([BirthDate]>='1930-01-01' AND [BirthDate]<=dateadd(year,(-18),getdate())))
GO

ALTER TABLE AdventureWorks..EmployeeNSet CHECK CONSTRAINT [CK_Employee_BirthDate]
GO

ALTER TABLE AdventureWorks..EmployeeNSet  WITH CHECK ADD  CONSTRAINT [CK_Employee_Gender] CHECK  ((upper([Gender])='F' OR upper([Gender])='M'))
GO

ALTER TABLE AdventureWorks..EmployeeNSet CHECK CONSTRAINT [CK_Employee_Gender]
GO

ALTER TABLE AdventureWorks..EmployeeNSet  WITH CHECK ADD  CONSTRAINT [CK_Employee_HireDate] CHECK  (([HireDate]>='1996-07-01' AND [HireDate]<=dateadd(day,(1),getdate())))
GO

ALTER TABLE AdventureWorks..EmployeeNSet CHECK CONSTRAINT [CK_Employee_HireDate]
GO

ALTER TABLE AdventureWorks..EmployeeNSet  WITH CHECK ADD  CONSTRAINT [CK_Employee_MaritalStatus] CHECK  ((upper([MaritalStatus])='S' OR upper([MaritalStatus])='M'))
GO

ALTER TABLE AdventureWorks..EmployeeNSet CHECK CONSTRAINT [CK_Employee_MaritalStatus]
GO

ALTER TABLE AdventureWorks..EmployeeNSet  WITH CHECK ADD  CONSTRAINT [CK_Employee_SickLeaveHours] CHECK  (([SickLeaveHours]>=(0) AND [SickLeaveHours]<=(120)))
GO

ALTER TABLE AdventureWorks..EmployeeNSet CHECK CONSTRAINT [CK_Employee_SickLeaveHours]
GO

ALTER TABLE AdventureWorks..EmployeeNSet  WITH CHECK ADD  CONSTRAINT [CK_Employee_VacationHours] CHECK  (([VacationHours]>=(-40) AND [VacationHours]<=(240)))
GO

ALTER TABLE AdventureWorks..EmployeeNSet CHECK CONSTRAINT [CK_Employee_VacationHours]
GO

ALTER TABLE AdventureWorks..EmployeeNSet ADD  CONSTRAINT [DF_Employee_SalariedFlag]  DEFAULT ((1)) FOR [SalariedFlag]
GO

ALTER TABLE AdventureWorks..EmployeeNSet ADD  CONSTRAINT [DF_Employee_VacationHours]  DEFAULT ((0)) FOR [VacationHours]
GO

ALTER TABLE AdventureWorks..EmployeeNSet ADD  CONSTRAINT [DF_Employee_SickLeaveHours]  DEFAULT ((0)) FOR [SickLeaveHours]
GO

ALTER TABLE AdventureWorks..EmployeeNSet ADD  CONSTRAINT [DF_Employee_CurrentFlag]  DEFAULT ((1)) FOR [CurrentFlag]
GO

ALTER TABLE AdventureWorks..EmployeeNSet ADD  CONSTRAINT [DF_Employee_rowguid]  DEFAULT (newid()) FOR [rowguid]
GO

ALTER TABLE AdventureWorks..EmployeeNSet ADD  CONSTRAINT [DF_Employee_ModifiedDate]  DEFAULT (getdate()) FOR [ModifiedDate]
GO

*/
