
USE AdventureWorks
GO

-- Add computed column and breadth-first index
ALTER TABLE HumanResources.NewOrg 
ADD H_level AS OrgNode.GetLevel() ;
CREATE UNIQUE INDEX EmpBFInd 
   ON HumanResources.NewOrg(H_level, OrgNode) ;
GO

-- employeeid is traditional lookup column
CREATE UNIQUE INDEX EmpIDs_unq ON HumanResources.NewOrg(EmployeeID) ;
GO

-- managerID is redundant
ALTER TABLE HumanResources.NewOrg DROP COLUMN ManagerID ;
GO

-- depth-first retrieval, entire tree
SELECT OrgNode.ToString() AS LogicalNode,
OrgNode, H_Level, EmployeeID, LoginID, Title
FROM HumanResources.NewOrg 
ORDER BY OrgNode;
GO

-- subtree 
-- all reports of employee 12 (node /1), two-levels deep
SELECT E.EmployeeID, E.LoginID
FROM HumanResources.NewOrg AS M
JOIN HumanResources.NewOrg AS E
  ON E.OrgNode.IsDescendantOf(M.OrgNode)=1
  AND M.EmployeeID = 12           
WHERE E.H_level - M.H_level <= 2

-- reverse subtree - all managers of employee 114 (node /1/1/4/3)
-- uses seek predicate and filter against DescendantLimit()
SELECT M.EmployeeID, M.LoginID
FROM HumanResources.NewOrg AS M
JOIN HumanResources.NewOrg AS E
  ON E.OrgNode.IsDescendantOf(M.OrgNode)=1
  AND E.EmployeeID = 114
  
-- direct subordinates of employee 158 (node /1/1/4)
-- uses DescendantLimit() in seek predicate
SELECT E.EmployeeID, E.LoginID
FROM HumanResources.NewOrg AS M
JOIN HumanResources.NewOrg AS E
  ON E.OrgNode.GetAncestor(1) = M.OrgNode
  AND M.EmployeeID = 158
  
SELECT E.EmployeeID, E.LoginID
FROM HumanResources.NewOrg AS M
JOIN HumanResources.NewOrg AS E WITH (Index = EmpBFInd)
  ON E.OrgNode.GetAncestor(1) = M.OrgNode
  AND M.EmployeeID = 158  

-- breadth-first retrieval, uses clustered index and sort
SELECT OrgNode.ToString() AS LogicalNode,
OrgNode, H_Level, EmployeeID, LoginID, Title
FROM HumanResources.NewOrg 
ORDER BY H_Level, OrgNode;
GO

-- retrieve by employeeid, uses clustered index and sort
SELECT OrgNode.ToString() AS LogicalNode,
OrgNode, H_Level, EmployeeID, LoginID, Title
FROM HumanResources.NewOrg 
ORDER BY EmployeeID;
GO

/*
-- employeeID could be redundant
-- it needs to be dropped for 32_GetDescendant examples
DROP INDEX EmpIDs_unq ON HumanResources.NewOrg ;
ALTER TABLE HumanResources.NewOrg DROP COLUMN EmployeeID ;
GO

*/