
USE AdventureWorks
GO

-- Move single row to new location
-- Wanida use to report to Sariya, now reports to Jill
DECLARE @CurrentEmployee hierarchyid , @OldParent hierarchyid, @NewParent hierarchyid
SELECT @CurrentEmployee = OrgNode FROM HumanResources.EmployeeOrg
  WHERE EmployeeID = 269 ; 
SELECT @OldParent = OrgNode FROM HumanResources.EmployeeOrg
  WHERE EmployeeID = 46 ; 
SELECT @NewParent = OrgNode FROM HumanResources.EmployeeOrg
  WHERE EmployeeID = 119 ; 

UPDATE HumanResources.EmployeeOrg
SET OrgNode = @CurrentEmployee. GetReparentedValue(@OldParent, @NewParent) 
WHERE OrgNode = @CurrentEmployee ;
GO

-- check result
SELECT OrgNode.ToString() AS Text_OrgNode, 
OrgNode, OrgLevel, EmployeeID, EmpName, Title 
FROM HumanResources.EmployeeOrg ;
GO

-- Add a new node
EXEC AddEmp 269, 291, 'Kevin', 'Marketing Intern'  ;
GO

-- check result, three-level hierarchy
SELECT OrgNode.ToString() AS Text_OrgNode, 
OrgNode, OrgLevel, EmployeeID, EmpName, Title 
FROM HumanResources.EmployeeOrg ;
GO

/*
-- Move all of Jill's team (but not Jill) to new manager, Sariya
-- This procedure assumes that the new manager has no direct reports before the move
-- else duplicate nodes may result
DECLARE @OldParent hierarchyid, @NewParent hierarchyid
SELECT @OldParent = OrgNode FROM HumanResources.EmployeeOrg
  WHERE EmployeeID = 119 ; -- Jill
SELECT @NewParent = OrgNode FROM HumanResources.EmployeeOrg
  WHERE EmployeeID = 46 ; -- Sariya

UPDATE HumanResources.EmployeeOrg
SET OrgNode = OrgNode. GetReparentedValue(@OldParent, @NewParent) 
WHERE 
OrgNode.IsDescendantOf(@OldParent) = 1 
AND EmployeeID <> 119 -- This excludes Jill from the move.
GO
*/

CREATE PROCEDURE MoveSubtree
  @empid AS INT,
  @new_mgrid AS INT
AS
DECLARE @old_root AS hierarchyid,  
        @new_root AS hierarchyid, 
        @new_mgr_hid AS hierarchyid

BEGIN TRAN
	SELECT @new_mgr_hid  = OrgNode 
	FROM HumanResources.EmployeeOrg WITH (UPDLOCK)
	WHERE EmployeeID = @new_mgrid   
	
	SELECT @old_root  = OrgNode 
	FROM HumanResources.EmployeeOrg 
	WHERE EmployeeID = @empid        
	
	-- Get a new hid for the root
	SET @new_root = @new_mgr_hid.GetDescendant(
		(SELECT MAX(OrgNode)
		 FROM HumanResources.EmployeeOrg 
		 WHERE OrgNode.GetAncestor(1) = @new_mgr_hid
		), NULL);
		
	UPDATE HumanResources.EmployeeOrg
	SET OrgNode = OrgNode. GetReparentedValue(@old_root, @new_root) 
	WHERE OrgNode.IsDescendantOf(@old_root) = 1 	
     
COMMIT TRAN
GO 

-- Move all of Jill's team (and Jill) to new manager, Sariya
EXEC MoveSubtree 119, 46

-- query again
SELECT OrgNode.ToString() AS Text_OrgNode, 
OrgNode, OrgLevel, EmployeeID, EmpName, Title 
FROM HumanResources.EmployeeOrg ;
GO


