
USE AdventureWorks
GO

SELECT OrgNode.ToString(), * FROM EmployeeHID

-- Find root
SELECT *
FROM EmployeeHID
WHERE OrgNode = HierarchyID::GetRoot()

-- Find leaves
SELECT *
FROM EmployeeHID AS Boss
WHERE NOT EXISTS 
 (SELECT * FROM EmployeeHID AS Worker
  WHERE Worker.OrgNode.GetAncestor(1) = Boss.OrgNode)

-- All subordinates for one boss, any level  
SELECT Mgrs.EmployeeID AS Boss, Workers.EmployeeID AS Worker
FROM EmployeeHID AS Mgrs, EmployeeHID AS Workers
WHERE Workers.OrgNode.IsDescendantOf(Mgrs.OrgNode)=1  
AND Mgrs.EmployeeID = 3

-- Subtrees
SELECT Mgrs.EmployeeID AS Boss, Workers.EmployeeID AS Worker
FROM EmployeeHID AS Mgrs, EmployeeHID AS Workers
WHERE Workers.OrgNode.IsDescendantOf(Mgrs.OrgNode)=1

-- or (not counting self)
SELECT Mgrs.EmployeeID AS Boss, Workers.EmployeeID AS Worker
FROM EmployeeHID AS Mgrs, EmployeeHID AS Workers
WHERE Workers.OrgNode.IsDescendantOf(Mgrs.OrgNode)=1
AND Workers.OrgNode <> Mgrs.OrgNode

-- Find level of each Worker
SELECT EmployeeID, OrgLevel
FROM EmployeeHID

-- Find height of a tree
SELECT MAX(OrgLevel)
FROM EmployeeHID

-- Levels Of Subordinates
-- Immediate Subordinates
SELECT Mgrs.EmployeeID AS Boss, Workers.EmployeeID AS Worker
FROM EmployeeHID AS Mgrs, EmployeeHID AS Workers
WHERE Workers.OrgNode.GetAncestor(1) = Mgrs.OrgNode
OR Workers.OrgNode = Mgrs.OrgNode

-- not counting self
SELECT Mgrs.EmployeeID AS Boss, Workers.EmployeeID AS Worker
FROM EmployeeHID AS Mgrs, EmployeeHID AS Workers
WHERE Workers.OrgNode.GetAncestor(1) = Mgrs.OrgNode

-- Generalize to any level of hierarchy
DECLARE @n int = 2
DECLARE @my_EmployeeID INT = 109
SELECT Workers.EmployeeID, 'is', @n, 'levels down from', @my_EmployeeID
FROM EmployeeHID AS Mgrs, EmployeeHID AS Workers
WHERE Workers.OrgNode.GetAncestor(@n) = Mgrs.OrgNode
AND Mgrs.EmployeeID = @my_EmployeeID
GO

-- all bosses for a specific employee
SELECT Mgrs.EmployeeID AS Boss, Workers.EmployeeID AS Worker
FROM EmployeeHID AS Mgrs, EmployeeHID AS Workers
WHERE Workers.OrgNode.IsDescendantOf(Mgrs.OrgNode)=1
AND Workers.EmployeeID = 114
AND Workers.EmployeeID <> Mgrs.EmployeeID

-- Oldest subordinate 
DECLARE @my_member INT = 109
SELECT TOP(1) Workers.EmployeeID, 'is the most senior subordinate of', @my_member
FROM EmployeeHID AS Mgrs, EmployeeHID AS Workers
WHERE Workers.OrgNode.GetAncestor(1) = Mgrs.OrgNode
AND Mgrs.EmployeeID = @my_member
ORDER BY Workers.OrgNode ASC
GO

-- Youngest subordinate
DECLARE @my_member INT = 109
SELECT TOP(1) Workers.EmployeeID, 'is the least senior subordinate of', @my_member
FROM EmployeeHID AS Mgrs, EmployeeHID AS Workers
WHERE Workers.OrgNode.GetAncestor(1) = Mgrs.OrgNode
AND Mgrs.EmployeeID = @my_member
ORDER BY Workers.OrgNode DESC
GO

-- nth sibling
DECLARE @n INT = 2
DECLARE @my_member INT = 109
SELECT S1.worker, 'is the', @n, '-th subordinate of', S1.boss
FROM (
SELECT Workers.EmployeeID as Worker, Mgrs.EmployeeID as Boss, ROW_NUMBER() OVER(ORDER BY Workers.OrgNode) AS num
FROM EmployeeHID AS Mgrs, EmployeeHID AS Workers
WHERE Workers.OrgNode.GetAncestor(1) = Mgrs.OrgNode
AND Mgrs.EmployeeID = @my_member ) AS S1
WHERE num = @n

-- complete list of siblings
SELECT Mgrs.EmployeeID as Boss, Workers.EmployeeID as Worker, 
	   ROW_NUMBER() OVER(PARTITION BY Mgrs.OrgNode ORDER BY Workers.OrgNode ) AS num
FROM EmployeeHID AS Mgrs, EmployeeHID AS Workers
WHERE Workers.OrgNode.GetAncestor(1) = Mgrs.OrgNode
GO

-- finding relative position
