
USE AdventureWorks
GO

SELECT * FROM EmployeeNSet

-- Find root
SELECT * 
FROM EmployeeNSet
WHERE lft = 1

-- Find leaves
SELECT * 
FROM EmployeeNSet
WHERE lft = rgt - 1

-- All subordinates for one boss, any level
SELECT Mgrs.EmployeeID AS Boss, Workers.EmployeeID AS worker
FROM EmployeeNSet AS Mgrs, EmployeeNSet As Workers
WHERE Workers.lft BETWEEN Mgrs.lft AND Mgrs.rgt
AND Mgrs.EmployeeID = 3 -- roberto

-- Subtrees
SELECT Mgrs.EmployeeID AS Boss, Workers.EmployeeID AS worker
FROM EmployeeNSet AS Mgrs, EmployeeNSet As Workers
WHERE Workers.lft BETWEEN Mgrs.lft AND Mgrs.rgt

-- or (not counting self)
SELECT Mgrs.EmployeeID AS Boss, Workers.EmployeeID AS worker
FROM EmployeeNSet AS Mgrs, EmployeeNSet As Workers
WHERE Workers.lft > Mgrs.lft AND Workers.lft < Mgrs.rgt

-- Find level of each worker
SELECT O2.EmployeeID, (COUNT(O2.EmployeeID) - 1) AS level
FROM EmployeeNSet AS O1, EmployeeNSet AS O2
WHERE O2.lft BETWEEN O1.lft AND O1.rgt
GROUP BY O2.EmployeeID

-- Find height of a tree
SELECT MAX(level) AS Height
FROM (SELECT O2.EmployeeID, (COUNT(O1.EmployeeID) - 1) AS level
      FROM EmployeeNSet AS O1, EmployeeNSet AS O2
	  WHERE O2.lft BETWEEN O1.lft AND O1.rgt
	  GROUP BY O2.EmployeeID) AS L1
GO
	  
-- Levels Of Subordinates
CREATE VIEW Immediate_Subordinates (boss, worker, lft, rgt)
AS
SELECT Mgrs.EmployeeID AS Boss, Workers.EmployeeID AS worker, Workers.lft, Workers.rgt
FROM EmployeeNSet AS Mgrs, EmployeeNSet As Workers
WHERE Workers.lft BETWEEN Mgrs.lft AND Mgrs.rgt
AND NOT EXISTS -- no middle manager 
  (SELECT *
   FROM EmployeeNSet AS MidMgr
   WHERE MidMgr.lft BETWEEN Mgrs.lft AND Mgrs.rgt
   AND Workers.lft  BETWEEN MidMgr.lft AND MidMgr.rgt
   AND MidMgr.EmployeeID NOT IN (Workers.EmployeeID, Mgrs.EmployeeID))
GO

-- use view   
SELECT * FROM Immediate_Subordinates;

-- not counting self
SELECT * FROM Immediate_Subordinates
WHERE boss <> worker;

-- Generalize to any level of hierarchy
DECLARE @n INT = 2
DECLARE @my_member INT = 109
SELECT Workers.EmployeeID, 'is', @n, 'levels down from', @my_member
FROM EmployeeNSet As Mgrs, EmployeeNSet As Workers
WHERE Mgrs.EmployeeID = @my_member
AND Workers.lft BETWEEN Mgrs.lft AND Mgrs.rgt
AND @n = (SELECT COUNT(MidMgr.EmployeeID) + 1
          FROM EmployeeNSet AS MidMgr
          WHERE MidMgr.lft BETWEEN Mgrs.lft AND Mgrs.rgt
          AND Workers.lft BETWEEN MidMgr.lft AND MidMgr.rgt
          AND MidMgr.EmployeeID NOT IN (Workers.EmployeeID, Mgrs.EmployeeID))
GO
          
-- Flattened version
DECLARE @n INT = 2
DECLARE @my_member INT = 109
SELECT Workers.EmployeeID, 'is', @n, 'levels down from', @my_member
FROM EmployeeNSet As Mgrs, EmployeeNSet As Workers, EmployeeNSet As MidMgr
WHERE Mgrs.EmployeeID = @my_member
AND Workers.lft BETWEEN Mgrs.lft AND Mgrs.rgt
AND MidMgr.lft BETWEEN Mgrs.lft AND Mgrs.rgt
AND Workers.lft BETWEEN MidMgr.lft AND MidMgr.rgt
AND MidMgr.EmployeeID NOT IN (Workers.EmployeeID, Mgrs.EmployeeID)
GROUP BY Workers.EmployeeID
HAVING @n = COUNT(MidMgr.EmployeeID) + 1
GO

-- all bosses for a specific employee
DECLARE @orig_worker INT = 114
SELECT Mgrs.EmployeeID AS Boss, Workers.EmployeeID AS worker
FROM EmployeeNSet AS Mgrs, EmployeeNSet As Workers
WHERE Workers.lft > Mgrs.lft AND Workers.lft < Mgrs.rgt
AND Workers.EmployeeID = @orig_worker

-- Oldest subordinate
DECLARE @my_member INT = 109
SELECT Workers.EmployeeID, 'is the most senior subordinate of', @my_member
FROM EmployeeNSet As Mgrs, EmployeeNSet As Workers
WHERE Mgrs.EmployeeID = @my_member
AND Workers.lft = Mgrs.lft + 1 -- leftmost child
GO

-- Youngest subordinate
DECLARE @my_member INT = 109
SELECT Workers.EmployeeID, 'is the least senior subordinate of', @my_member
FROM EmployeeNSet As Mgrs, EmployeeNSet As Workers
WHERE Mgrs.EmployeeID = @my_member
AND Workers.rgt = Mgrs.rgt - 1 -- rightmost child

-- Find the nth sibling
DECLARE @n INT = 2
DECLARE @my_member INT = 109;
SELECT S1.worker, 'is the', @n, '-th subordinate of', S1.boss
FROM Immediate_Subordinates AS S1
WHERE S1.boss = @my_member
AND @n = (SELECT COUNT(S2.lft) - 1
          FROM Immediate_Subordinates As S2
          WHERE S2.boss = S1.boss
          AND S2.boss <> S1.worker
          AND S2.lft BETWEEN 1 AND S1.lft)
          
-- Or complete listing of siblings
SELECT O1.EmployeeID AS boss, S1.worker,
       COUNT(S2.lft) AS sibling_order
FROM Immediate_Subordinates AS S1,
	 Immediate_Subordinates AS S2,
	 EmployeeNSet AS O1
WHERE S1.boss = O1.EmployeeID
AND S2.boss = S1.boss
AND S1.worker <> S2.worker
AND S2.lft <= S1.lft
GROUP BY O1.EmployeeID, S1.worker
GO

-- finding relative position
DECLARE @first_member INT = 109
DECLARE @first_member_c CHAR(3) = '109'
DECLARE @second_member INT = 114
DECLARE @second_member_c CHAR(3) = '114'
SELECT CASE WHEN @first_member = @second_member
			  THEN @first_member_c + ' is ' + @second_member_c
			WHEN O1.lft BETWEEN O2.lft AND O2.rgt
			  THEN @first_member_c + ' subordinate to ' + @second_member_c
			WHEN O2.lft BETWEEN O1.lft AND O1.rgt
			  THEN @second_member_c + ' subordinate to ' + @first_member_c
			ELSE	
			  @second_member_c + ' no relation to ' + @first_member_c
	   END		  
FROM EmployeeNSet AS O1, EmployeeNSet AS O2
WHERE O1.EmployeeID = @first_member
AND   O2.EmployeeID = @second_member

-- get level by calculating lft/rgt groupings between nodes
DECLARE @my_member INT = 114
SELECT @my_member, COUNT(Mgrs.EmployeeID) AS level
FROM EmployeeNSet AS Mgrs, EmployeeNSet AS Workers
WHERE Workers.lft BETWEEN Mgrs.lft AND Mgrs.rgt
AND Workers.EmployeeID = @my_member