
CREATE TABLE OrgChartH (
 member CHAR(10) NOT NULL PRIMARY KEY,
 hid HierarchyID
)
GO

INSERT OrgChartH VALUES ('Albert', HierarchyID::Parse('/'))
INSERT OrgChartH VALUES ('Bert', HierarchyID::Parse('/1/'))
INSERT OrgChartH VALUES ('Chuck', HierarchyID::Parse('/2/'))
INSERT OrgChartH VALUES ('Donna', HierarchyID::Parse('/2/1/'))
INSERT OrgChartH VALUES ('Eddie', HierarchyID::Parse('/2/2/'))
INSERT OrgChartH VALUES ('Fred', HierarchyID::Parse('/2/3/'))
GO

SELECT member, hid.ToString() as HierPos 
FROM OrgChartH

-- Find root
SELECT *
FROM OrgChartH
WHERE hid = HierarchyID::GetRoot()

-- Subtrees
SELECT Mgrs.member AS Boss, Workers.member AS worker
FROM OrgChartH AS Mgrs, OrgChartH AS Workers
WHERE Workers.hid.IsDescendantOf(Mgrs.hid)=1

-- or (not counting self)
SELECT Mgrs.member AS Boss, Workers.member AS worker
FROM OrgChartH AS Mgrs, OrgChartH AS Workers
WHERE Workers.hid.IsDescendantOf(Mgrs.hid)=1
AND Workers.hid <> Mgrs.hid

-- Find level of each worker
SELECT member, hid.GetLevel()
FROM OrgChartH

-- Find height of a tree
SELECT MAX(hid.GetLevel())
FROM OrgChartH

-- Levels Of Subordinates
-- Immediate Subordinates
SELECT Mgrs.member AS Boss, Workers.member AS worker
FROM OrgChartH AS Mgrs, OrgChartH AS Workers
WHERE Workers.hid.GetAncestor(1) = Mgrs.hid
OR Workers.hid = Mgrs.hid

-- not counting self
SELECT Mgrs.member AS Boss, Workers.member AS worker
FROM OrgChartH AS Mgrs, OrgChartH AS Workers
WHERE Workers.hid.GetAncestor(1) = Mgrs.hid

-- Generalize to any level of hierarchy
DECLARE @n int = 2
DECLARE @my_member CHAR(10) = 'Albert'
SELECT Workers.member, 'is', @n, 'levels down from', @my_member
FROM OrgChartH AS Mgrs, OrgChartH AS Workers
WHERE Workers.hid.GetAncestor(@n) = Mgrs.hid
AND Mgrs.member = @my_member
GO

-- Another table

DROP TABLE AssembliesH
GO

CREATE TABLE AssembliesH (
  part CHAR(2) NOT NULL,
  hid HierarchyID UNIQUE
)
GO

INSERT AssembliesH 
VALUES	('A', HierarchyID::Parse('/')),
		('B', HierarchyID::Parse('/1/')),
		('C', HierarchyID::Parse('/2/')),
		('D', HierarchyID::Parse('/3/')),
		('E', HierarchyID::Parse('/1/1/')),
		('F', HierarchyID::Parse('/2/1/')),
		('G', HierarchyID::Parse('/2/2/')),
		('H', HierarchyID::Parse('/3/1/')),
		('I', HierarchyID::Parse('/2/1/1/')),
		('J', HierarchyID::Parse('/2/1/2/')),
		('K', HierarchyID::Parse('/3/1/1/')),
		('L', HierarchyID::Parse('/3/1/2/')),
		('M', HierarchyID::Parse('/2/1/2/1/')),
		('N', HierarchyID::Parse('/2/1/2/2/'));
GO

-- Show Levels as a single row
-- Not quite, shows hid, not part number 
SELECT PART,
	   hid.GetAncestor(1),
	   hid.GetAncestor(2),
	   hid.GetAncestor(3),
	   hid.GetAncestor(4)   
FROM AssembliesH

SELECT PART,
	   (SELECT part from AssembliesH WHERE hid = A.hid.GetAncestor(1)) as level_0,
	   (SELECT part from AssembliesH WHERE hid = A.hid.GetAncestor(2)) as level_1,
	   (SELECT part from AssembliesH WHERE hid = A.hid.GetAncestor(3)) as level_2,
	   (SELECT part from AssembliesH WHERE hid = A.hid.GetAncestor(4)) as level_3   
FROM AssembliesH A

-- Oldest subordinate (better with max and min?)
DECLARE @my_member CHAR(10) = 'Albert'
SELECT TOP(1) Workers.member, 'is the most senior subordinate of', @my_member
FROM OrgChartH AS Mgrs, OrgChartH AS Workers
WHERE Workers.hid.GetAncestor(1) = Mgrs.hid
AND Mgrs.member = @my_member
GO

-- Youngest subordinate
DECLARE @my_member CHAR(10) = 'Albert'
SELECT TOP(1) Workers.member, 'is the least senior subordinate of', @my_member
FROM OrgChartH AS Mgrs, OrgChartH AS Workers
WHERE Workers.hid.GetAncestor(1) = Mgrs.hid
AND Mgrs.member = @my_member
ORDER BY Workers.hid DESC
GO

-- nth sibling
DECLARE @n INT = 2
DECLARE @my_member CHAR(10) = 'Albert'
SELECT S1.worker, 'is the', @n, '-th subordinate of', S1.boss
FROM (
SELECT Workers.member as Worker, Mgrs.member as Boss, ROW_NUMBER() OVER(ORDER BY Workers.hid) AS num
FROM OrgChartH AS Mgrs, OrgChartH AS Workers
WHERE Workers.hid.GetAncestor(1) = Mgrs.hid
AND Mgrs.member = @my_member ) AS S1
WHERE num = @n

-- complete list of siblings
SELECT Mgrs.member as Boss, Workers.member as Worker, 
	   ROW_NUMBER() OVER(PARTITION BY Mgrs.hid ORDER BY Workers.hid ) AS num
FROM OrgChartH AS Mgrs, OrgChartH AS Workers
WHERE Workers.hid.GetAncestor(1) = Mgrs.hid
GO

-- find all of your siblings (including you) 
DECLARE @my_sibling_part CHAR(2) = 'G'
SELECT A3.part
FROM AssembliesH A1, AssembliesH A2, AssembliesH A3
WHERE A1.part = @my_sibling_part
AND A1.hid.GetAncestor(1) = A2.hid
AND A3.hid.GetLevel() = A1.hid.GetLevel()
AND A3.hid.IsDescendantOf(A2.hid)=1

-- finding a path

-- finding relative position
        
-- iterative parts update
CREATE TABLE FrammisH (
  part CHAR(2) PRIMARY KEY,
  qty INT NOT NULL,
  wgt INT NOT NULL,
  hid HierarchyID UNIQUE
);
GO

INSERT FrammisH 
VALUES	('A', 1, 0, HierarchyID::Parse('/')),
		('B', 1, 0, HierarchyID::Parse('/1/')),
		('C', 2, 0, HierarchyID::Parse('/2/')),
		('D', 2, 0, HierarchyID::Parse('/3/')),
		('E', 2, 12, HierarchyID::Parse('/1/1/')),
		('F', 5, 0, HierarchyID::Parse('/2/1/')),
		('G', 2, 6, HierarchyID::Parse('/2/2/')),
		('H', 3, 0, HierarchyID::Parse('/3/1/')),
		('I', 4, 8, HierarchyID::Parse('/2/1/1/')),
		('J', 1, 0, HierarchyID::Parse('/2/1/2/')),
		('K', 5, 3, HierarchyID::Parse('/3/1/1/')),
		('L', 1, 4, HierarchyID::Parse('/3/1/2/')),
		('M', 2, 7, HierarchyID::Parse('/2/1/2/1/')),
		('N', 3, 2, HierarchyID::Parse('/2/1/2/2/'));
GO

CREATE PROCEDURE WgtCalc_1H 
AS
BEGIN
-- ...
END;
GO

-- Recursive parts Update
CREATE FUNCTION WgtCalc_2H (@my_part CHAR(2))
RETURNS INT
AS
RETURN (SELECT ... )
GO

-- Inserting and updating Trees

-- Moving a subtree within a tree

-- Subtree duplication

-- Swapping siblings

-- Converting HierarchyID To Adjacency List

-- Converting Adjacency List to HierarchyID