
CREATE TABLE OrgChart (
 member CHAR(10) NOT NULL PRIMARY KEY,
 lft INT NOT NULL,
 rgt INT NOT NULL
)
GO

INSERT OrgChart VALUES ('Albert',1,12)
INSERT OrgChart VALUES ('Bert',2,3)
INSERT OrgChart VALUES ('Chuck',4,11)
INSERT OrgChart VALUES ('Donna',5,6)
INSERT OrgChart VALUES ('Eddie',7,8)
INSERT OrgChart VALUES ('Fred',9,10)
GO

-- Find root
SELECT * 
FROM OrgChart
WHERE lft = 1

-- Find leaves
SELECT * 
FROM OrgChart
WHERE lft = rgt - 1

-- Subtrees
SELECT Mgrs.member AS Boss, Workers.member AS worker
FROM OrgChart AS Mgrs, OrgChart As Workers
WHERE Workers.lft BETWEEN Mgrs.lft AND Mgrs.rgt

-- or (not counting self)
SELECT Mgrs.member AS Boss, Workers.member AS worker
FROM OrgChart AS Mgrs, OrgChart As Workers
WHERE Workers.lft > Mgrs.lft AND Workers.lft < Mgrs.rgt

-- Find level of each worker
SELECT O2.member, (COUNT(O2.member) - 1) AS level
FROM OrgChart AS O1, OrgChart AS O2
WHERE O2.lft BETWEEN O1.lft AND O1.rgt
GROUP BY O2.member

-- Find height of a tree
SELECT MAX(level) AS Height
FROM (SELECT O2.member, (COUNT(O1.member) - 1) AS level
      FROM OrgChart AS O1, OrgChart AS O2
	  WHERE O2.lft BETWEEN O1.lft AND O1.rgt
	  GROUP BY O2.member) AS L1
GO
	  
-- Levels Of Subordinates
CREATE VIEW Immediate_Subordinates (boss, worker, lft, rgt)
AS
SELECT Mgrs.member AS Boss, Workers.member AS worker, Workers.lft, Workers.rgt
FROM OrgChart AS Mgrs, OrgChart As Workers
WHERE Workers.lft BETWEEN Mgrs.lft AND Mgrs.rgt
AND NOT EXISTS -- no middle manager 
  (SELECT *
   FROM OrgChart AS MidMgr
   WHERE MidMgr.lft BETWEEN Mgrs.lft AND Mgrs.rgt
   AND Workers.lft  BETWEEN MidMgr.lft AND MidMgr.rgt
   AND MidMgr.member NOT IN (Workers.member, Mgrs.member))
GO
   
SELECT * FROM Immediate_Subordinates;

-- Generalize to any level of hierarchy
DECLARE @n INT = 2
DECLARE @my_member CHAR(10) = 'Albert'
SELECT Workers.member, 'is', @n, 'levels down from', @my_member
FROM OrgChart As Mgrs, OrgChart As Workers
WHERE Mgrs.member = @my_member
AND Workers.lft BETWEEN Mgrs.lft AND Mgrs.rgt
AND @n = (SELECT COUNT(MidMgr.member) + 1
          FROM OrgChart AS MidMgr
          WHERE MidMgr.lft BETWEEN Mgrs.lft AND Mgrs.rgt
          AND Workers.lft BETWEEN MidMgr.lft AND MidMgr.rgt
          AND MidMgr.member NOT IN (Workers.member, Mgrs.member))
GO
          
-- Flattened version
DECLARE @n INT = 2
DECLARE @my_member CHAR(10) = 'Albert'
SELECT Workers.member, 'is', @n, 'levels down from', @my_member
FROM OrgChart As Mgrs, OrgChart As Workers, OrgChart As MidMgr
WHERE Mgrs.member = @my_member
AND Workers.lft BETWEEN Mgrs.lft AND Mgrs.rgt
AND MidMgr.lft BETWEEN Mgrs.lft AND Mgrs.rgt
AND Workers.lft BETWEEN MidMgr.lft AND MidMgr.rgt
AND MidMgr.member NOT IN (Workers.member, Mgrs.member)
GROUP BY Workers.member
HAVING @n = COUNT(MidMgr.member) + 1
GO

-- Another table
CREATE TABLE Assemblies (
  part CHAR(2) NOT NULL,
  lft INT NOT NULL,
  rgt INT NOT NULL
)
GO

INSERT Assemblies 
VALUES	('A', 1, 28),
		('B', 2, 5),
		('C', 6, 19),
		('D', 20, 27),
		('E', 3, 4),
		('F', 7, 16),
		('G', 17, 18),
		('H', 21, 26),
		('I', 8, 9),
		('J', 10, 15),
		('K', 22, 23),
		('L', 24, 25),
		('M', 11, 12),
		('N', 13, 14);
GO

-- Show Levels as a single row
CREATE VIEW Flat_Parts(part, level_0, level_1, level_2, level_3)
AS
SELECT A1.part,
       CASE WHEN COUNT(A3.part) = 2
	        THEN A2.part
	        ELSE NULL END AS level_0,
       CASE WHEN COUNT(A3.part) = 3
	        THEN A2.part
	        ELSE NULL END AS level_1,
       CASE WHEN COUNT(A3.part) = 4
	        THEN A2.part
	        ELSE NULL END AS level_2,	
       CASE WHEN COUNT(A3.part) = 5
	        THEN A2.part
	        ELSE NULL END AS level_3        
FROM Assemblies AS A1,
	 Assemblies AS A2,
	 Assemblies AS A3
WHERE A1.lft BETWEEN A2.lft AND A2.rgt
AND   A3.lft BETWEEN A2.lft AND A2.rgt
AND   A1.lft BETWEEN A3.lft AND A3.rgt
GROUP BY A1.part, A2.part;
GO

SELECT part, MAX(level_0), MAX(level_1), MAX(level_2), MAX(level_3)
FROM Flat_Parts
GROUP BY part;

-- folded as one query
SELECT A1.part,
	   (SELECT part
	    FROM Assemblies
	    WHERE lft = MAX(A2.lft)) AS level_0,
	   (SELECT part
	    FROM Assemblies
	    WHERE lft = MAX(A3.lft)) AS level_1,
	   (SELECT part
	    FROM Assemblies
	    WHERE lft = MAX(A4.lft)) AS level_2,
	   (SELECT part
	    FROM Assemblies
	    WHERE lft = MAX(A5.lft)) AS level_3
FROM Assemblies AS A1
LEFT OUTER JOIN Assemblies AS A2
  ON A1.lft > A2.lft AND A1.rgt < A2.rgt
LEFT OUTER JOIN Assemblies AS A3
  ON A2.lft > A3.lft AND A2.rgt < A3.rgt
LEFT OUTER JOIN Assemblies AS A4
  ON A3.lft > A4.lft AND A3.rgt < A4.rgt
LEFT OUTER JOIN Assemblies AS A5
  ON A4.lft > A5.lft AND A4.rgt < A5.rgt
GROUP BY A1.part
GO

-- Oldest subordinate
DECLARE @my_member CHAR(10) = 'Albert'
SELECT Workers.member, 'is the most senior subordinate of', @my_member
FROM OrgChart As Mgrs, OrgChart As Workers
WHERE Mgrs.member = @my_member
AND Workers.lft = Mgrs.lft + 1 -- leftmost child
GO

-- Youngest subordinate
DECLARE @my_member CHAR(10) = 'Albert'
SELECT Workers.member, 'is the least senior subordinate of', @my_member
FROM OrgChart As Mgrs, OrgChart As Workers
WHERE Mgrs.member = @my_member
AND Workers.rgt = Mgrs.rgt - 1 -- rightmost child

-- Find the nth sibling
DECLARE @n INT = 2
DECLARE @my_member CHAR(10) = 'Albert'
SELECT S1.worker, 'is the', @n, '-th subordinate of', S1.boss
FROM Immediate_Subordinates AS S1
WHERE S1.boss = @my_member
AND @n = (SELECT COUNT(S2.lft) - 1
          FROM Immediate_Subordinates As S2
          WHERE S2.boss = S1.boss
          AND S2.boss <> S1.worker
          AND S2.lft BETWEEN 1 AND S1.lft)
          
-- Or complete listing of siblings
SELECT O1.member AS boss, S1.worker,
       COUNT(S2.lft) AS sibling_order
FROM Immediate_Subordinates AS S1,
	 Immediate_Subordinates AS S2,
	 OrgChart AS O1
WHERE S1.boss = O1.member
AND S2.boss = S1.boss
AND S1.worker <> S2.worker
AND S2.lft <= S1.lft
GROUP BY O1.member, S1.worker
GO

-- siblings of a given node with a level number
CREATE VIEW Siblings (lvl, part, lft, rgt)
AS
SELECT COUNT(A2.lft), A1.part, A1.lft, A1.rgt
FROM Assemblies AS A1, Assemblies AS A2
WHERE A1.lft BETWEEN A2.lft AND A2.rgt
GROUP BY A1.part, A1.lft, A1.rgt

SELECT * FROM Siblings

-- use the view, to find all of your siblings (including you) 
DECLARE @my_sibling_part CHAR(2) = 'G'
SELECT DISTINCT S2.part
FROM Siblings AS S1, Siblings AS S2
WHERE S1.part = @my_sibling_part
AND EXISTS 
    (SELECT * 
     FROM Siblings AS S0
     WHERE S1.lft BETWEEN S0.lft AND S0.rgt
     AND   S2.lft BETWEEN S0.lft AND S0.rgt
     AND   S0.lvl = S1.lvl - 1
     AND   S1.lvl = S2.lvl);
     
-- finding a path 
DECLARE @start_node CHAR(2) = 'C'
DECLARE @finish_node CHAR(2) = 'N'
SELECT A2.part,
       (SELECT COUNT(*) 
        FROM   Assemblies AS A4
        WHERE  A4.lft BETWEEN A1.lft AND A1.rgt
        AND    A2.lft BETWEEN A4.lft AND A4.rgt) AS path_nbr
FROM Assemblies AS A1, Assemblies AS A2, Assemblies AS A3
WHERE A1.part = @start_node 
AND   A3.part = @finish_node
AND   A2.lft BETWEEN A1.lft and A1.rgt
AND   A3.lft BETWEEN A2.lft AND A2.rgt
GO

-- finding relative position
DECLARE @first_member CHAR(10) = 'Albert'
DECLARE @second_member CHAR(10) = 'Donna'
SELECT CASE WHEN @first_member = @second_member
			  THEN @first_member + ' is ' + @second_member
			WHEN O1.lft BETWEEN O2.lft AND O2.rgt
			  THEN @first_member + ' subordinate to ' + @second_member
			WHEN O2.left BETWEEN O1.lft AND O1.rgt
			  THEN @second_member + ' subordinate to ' + @first_member
			ELSE	
			  @second_member + ' no relation to ' + @first_member
	   END		  
FROM OrgChart AS O1, OrgChart AS O2
WHERE O1.member = @first_member
AND   O2.member = @second_member

-- get level by calculating lft/rgt groupings between nodes
DECLARE @my_member CHAR(10) = 'Donna'
SELECT @my_member, COUNT(Mgrs.Member) AS level
FROM OrgChart AS Mgrs, OrgChart AS Workers
WHERE Workers.lft BETWEEN Mgrs.lft AND Mgrs.rgt
AND Workers.member = @my_member

-- Pyramid sales operation
CREATE TABLE Sales (
  member CHAR(10) NOT NULL PRIMARY KEY,
  sale_amt DECIMAL(12,4) NOT NULL);
GO  

-- No personnel table defined
DECLARE @my_member CHAR(10) = 'Donna'
SELECT @my_member, SUM(S1.sale_amt) AS total_sales
FROM OrgChart AS Mgrs, OrgChart AS Workers,
	 Sales AS S1
WHERE Workers.lft BETWEEN Mgrs.lft AND Mgrs.rgt
AND P1.job_title = Workers.job_title
AND Mgrs.member = @my_member;	 

-- Accumulated total - no blueprint table
SELECT @this_part, SUM(Subassembly.qty * Subassem.price) AS totalcost
FROM Blueprint AS Assembly, Blueprint AS Subassem
WHERE Subassem.lft BETWEEN Assembly.lft and Assembly.rgt
AND Assembly.part = @this_part

-- Skipped deleting examples

-- Iterative parts update
CREATE TABLE Frammis (
  part CHAR(2) PRIMARY KEY,
  qty INT NOT NULL,
  wgt INT NOT NULL,
  lft INT NOT NULL
    CONSTRAINT valid_lft CHECK (lft > 0),
  rgt INT NOT NULL
    CONSTRAINT valid_rgt CHECK (rgt > 0),
  CONSTRAINT valid_range_pair CHECK (lft < rgt)
);

INSERT Frammis VALUES
  ('A', 1, 0, 1, 28),  
  ('B', 1, 0, 2, 5),
  ('C', 2, 0, 6, 19),
  ('D', 2, 0, 20, 27),
  ('E', 2, 12, 3, 4),
  ('F', 5, 0, 7, 16),
  ('G', 2, 6, 17, 18),
  ('H', 3, 0, 21, 26),
  ('I', 4, 8, 8, 9),
  ('J', 1, 0, 10, 15),
  ('K', 5, 3, 22, 23),
  ('L', 1, 4, 24, 25),
  ('M', 2, 7, 11, 12),
  ('N', 3, 2, 13, 14);
GO

CREATE TABLE Parts (
  part_id CHAR(2) NOT NULL PRIMARY KEY,
  part_name VARCHAR(15) NOT NULL,
  wgt INT NOT NULL CHECK (wgt >= 0),
  supplier_nbr INT NOT NULL
);
GO

CREATE PROCEDURE WgtCalc_1 
AS
BEGIN
-- ...
END;
GO

-- Recursive parts Update
CREATE FUNCTION WgtCalc_2 (@my_part CHAR(2))
RETURNS INT
AS
RETURN (SELECT ... )
GO

-- Inserting and updating Trees

-- Moving a subtree within a tree

-- Move subtree, second version

-- Subtree duplication

-- Swapping siblings

-- Converting Nested Set To Adjacency List

-- Converting Adjacency List to Nested Set